#ifndef S2HAT_CUDA_COMMON_CUH
#define S2HAT_CUDA_COMMON_CUH

// File: s2hat_cuda_common.cuh
// Created by Ovidiu Hupca, Mikolaj Szydlarski, Giulio Fabbian on 2012-04-26.
// 
// Copyright (C) 2012 Joel Falcou, Laura Grigori, Ovidiu Hupca, Radek Stompor, Mikolaj Szydlarski
// 
// This file is part of S2HAT_CUDA.
// 
// S2HAT_CUDA is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// S2HAT_CUDA is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with S2HAT_CUDA (see 'S2HAT_CUDA_LICENSE.txt' file). 
// If not, see <http://www.gnu.org/licenses/>.

// ==========================================================================================
// = Main configuration file for CUDA routines = 
//  
//  DESCRIPTION:
//		HARD_VERBOSE: increase level of verbosity.
//		PREFER_L1: prefer larger L1 cache and smaller shared memory. Only works on Fermi or Tesla cards
//		CUFFT: use CUDA fft instead of fftw
//		KERNEL_TIMER: measure the GPU kernel execution time.
//		NR_THREADS_PER_BLOCK( _A2M or _M2A) : set maximum number of threads per block in alm2map (_A2M) or map2alm (_M2A) routine (i.e. rings)
//		RECFAC_CHUNK_SIZE(_A2M or _M2A) : size of chunk in refactorization table in alm2map (_A2M) or map2alm (_M2A) routine
//		ALM_CHUNK_SIZE(_A2M or _M2A): size of chunks for tables with alm coefficients in alm2map (_A2M) or map2alm (_M2A) routine
// ==========================================================================================

// !! (un)comment if needed 

// #define HARD_VERBOSE

// #define PREFER_L1
#define CUFFT
#define KERNEL_TIMER 

#ifndef EXEC_CONFIG
	
	#define NR_THREADS_PER_BLOCK_A2M 128
	#define NR_THREADS_PER_BLOCK_M2A  64
	
	// [!] CUDA thread management WARNING: ALM_CHUNK_SIZE and RECFAC_CHUNK_SIZE must be a multiple of NR_THREADS_PER_BLOCK_A2M !!
	
	#define RECFAC_CHUNK_SIZE_A2M (2 * NR_THREADS_PER_BLOCK_A2M)
	#define ALM_CHUNK_SIZE_A2M (2 * NR_THREADS_PER_BLOCK_A2M)
	
	#define RECFAC_CHUNK_SIZE_M2A (2 * NR_THREADS_PER_BLOCK_M2A)
	#define ALM_CHUNK_SIZE_M2A (2 * NR_THREADS_PER_BLOCK_M2A)
	
#endif

#define REAL_MAX DBL_MAX
#define SMAX 10
#define RESCALE_TAB_SIZE (2*SMAX+1)
#define RESCALE_TAB_S(X)  ( rescale_tab[ (X + SMAX) ] )

// ==========================================================================================

#include "s2hat_cuda_tools.cuh"

#include <mpi.h>
#include <cuda.h>
#include <cufft.h>
#include <cuComplex.h>
#include <cuda_runtime.h>

#ifdef __cplusplus
	extern "C" {
#endif

#include "s2hat_utils.h"
#include "s2hat_c_common.h"

#ifdef __cplusplus
	}
#endif

#include "s2hat_defs.h"
#include "s2hat.h"

struct cufft_counters_c2r {
	int cufft_c2r_plan_set;
	int cufft_call_count;
	int cufft_full_count;
	
	double complex* fft_vec_in;

	int cufft_c2r_plan;
};

struct cufft_counters_r2c {
	int cufft_r2c_plan_set;
	int cufft_call_count;
	int cufft_full_count;
	
	double complex* fft_vec_out;

	int cufft_r2c_plan;
};

#endif
