#ifndef S2HAT_UTILS
#define S2HAT_UTILS

// File: s2hat_utils.h
// Created by Ovidiu Hupca and Mikolaj Szydlarski, Giulio Fabbian on 2012-04-26.
// 
// Copyright (C) 2012 Joel Falcou, Laura Grigori, Ovidiu Hupca, Radek Stompor, Mikolaj Szydlarski
// 
// This file is part of S2HAT_CUDA.
// 
// S2HAT_CUDA is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// S2HAT_CUDA is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with S2HAT_CUDA (see 'S2HAT_CUDA_LICENSE.txt' file). 
// If not, see <http://www.gnu.org/licenses/>.

#include "string.h"
#include "stdlib.h"
#include "stdio.h"
#include "stdint.h"
#include "time.h"
#include "math.h"

#include "sys/types.h"
#include "sys/stat.h"
#include "sys/param.h"

#include "s2hat.h"

void *s2hat_malloc(size_t sz);
void s2hat_free(void *ptr);

#define S2HATALLOC(ptr,type,num) \
  do { (ptr)=(type *)s2hat_malloc((num)*sizeof(type)); } while (0)

double timespecDiff(clock_t* stopm, clock_t* startm);
double drand (double min, double max);

void s2hat_random_alm (s2hat_dcomplex* alm, int* mvals, int mmax, int nlmax);
void s2hat_print_alm (s2hat_dcomplex* alm, int* mvals, int mmax, int nlmax);
void s2hat_print_map (s2hat_flt8* map, int map_size);

void s2hat_print_cuda_times(char * title, s2hat_cuda_times* ct, int proces_id);
void s2hat_print_cuda_min_max_times(char * title, s2hat_cuda_times* ct, int proces_id);
void s2hat_fprint_cuda_min_max_times(char * title, s2hat_cuda_times* ct, int proces_id,double dt,int nprocs);

void s2hat_print_mt_times(char * title, s2hat_mt_times* ct, int proces_id);
void s2hat_print_mt_min_max_times(char * title, s2hat_mt_times* ct, int proces_id);
void s2hat_fprint_mt_min_max_times(char * title, s2hat_mt_times* ct, int proces_id,double dt,int nprocs);

void s2hat_measure_errors (s2hat_dcomplex* alm_init, s2hat_dcomplex* alm_final, int nalms, int proces_id);

#define S2HATALLOC(ptr,type,num) \
  do { (ptr)=(type *)s2hat_malloc((num)*sizeof(type)); } while (0) // adapted from "libpsht by Martin Reinecke"

#endif